

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

Represents a precise time, with seconds and microseconds.
Similar to the struct timeval returned by the @/gettimeofday()/@
UNIX system call.

GLib is attempting to unify around the use of 64bit integers to
represent microsecond-precision time. As such, this type will be
removed from a future version of GLib. A consequence of using @glong@ for
@tv_sec@ is that on 32-bit systems @GTimeVal@ is subject to the year 2038
problem.
-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.GLib.Structs.TimeVal
    ( 

-- * Exported types
    TimeVal(..)                             ,
    newZeroTimeVal                          ,
    noTimeVal                               ,


 -- * Methods
-- ** add #method:add#

#if ENABLE_OVERLOADING
    TimeValAddMethodInfo                    ,
#endif
    timeValAdd                              ,


-- ** fromIso8601 #method:fromIso8601#

    timeValFromIso8601                      ,


-- ** toIso8601 #method:toIso8601#

#if ENABLE_OVERLOADING
    TimeValToIso8601MethodInfo              ,
#endif
    timeValToIso8601                        ,




 -- * Properties
-- ** tvSec #attr:tvSec#
{- | seconds
-}
    getTimeValTvSec                         ,
    setTimeValTvSec                         ,
#if ENABLE_OVERLOADING
    timeVal_tvSec                           ,
#endif


-- ** tvUsec #attr:tvUsec#
{- | microseconds
-}
    getTimeValTvUsec                        ,
    setTimeValTvUsec                        ,
#if ENABLE_OVERLOADING
    timeVal_tvUsec                          ,
#endif




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL


-- | Memory-managed wrapper type.
newtype TimeVal = TimeVal (ManagedPtr TimeVal)
instance WrappedPtr TimeVal where
    wrappedPtrCalloc = callocBytes 16
    wrappedPtrCopy = \p -> withManagedPtr p (copyBytes 16 >=> wrapPtr TimeVal)
    wrappedPtrFree = Just ptr_to_g_free

-- | Construct a `TimeVal` struct initialized to zero.
newZeroTimeVal :: MonadIO m => m TimeVal
newZeroTimeVal = liftIO $ wrappedPtrCalloc >>= wrapPtr TimeVal

instance tag ~ 'AttrSet => Constructible TimeVal tag where
    new _ attrs = do
        o <- newZeroTimeVal
        GI.Attributes.set o attrs
        return o


-- | A convenience alias for `Nothing` :: `Maybe` `TimeVal`.
noTimeVal :: Maybe TimeVal
noTimeVal = Nothing

{- |
Get the value of the “@tv_sec@” field.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.get' timeVal #tvSec
@
-}
getTimeValTvSec :: MonadIO m => TimeVal -> m CLong
getTimeValTvSec s = liftIO $ withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO CLong
    return val

{- |
Set the value of the “@tv_sec@” field.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.set' timeVal [ #tvSec 'Data.GI.Base.Attributes.:=' value ]
@
-}
setTimeValTvSec :: MonadIO m => TimeVal -> CLong -> m ()
setTimeValTvSec s val = liftIO $ withManagedPtr s $ \ptr -> do
    poke (ptr `plusPtr` 0) (val :: CLong)

#if ENABLE_OVERLOADING
data TimeValTvSecFieldInfo
instance AttrInfo TimeValTvSecFieldInfo where
    type AttrAllowedOps TimeValTvSecFieldInfo = '[ 'AttrSet, 'AttrGet]
    type AttrSetTypeConstraint TimeValTvSecFieldInfo = (~) CLong
    type AttrBaseTypeConstraint TimeValTvSecFieldInfo = (~) TimeVal
    type AttrGetType TimeValTvSecFieldInfo = CLong
    type AttrLabel TimeValTvSecFieldInfo = "tv_sec"
    type AttrOrigin TimeValTvSecFieldInfo = TimeVal
    attrGet _ = getTimeValTvSec
    attrSet _ = setTimeValTvSec
    attrConstruct = undefined
    attrClear _ = undefined

timeVal_tvSec :: AttrLabelProxy "tvSec"
timeVal_tvSec = AttrLabelProxy

#endif


{- |
Get the value of the “@tv_usec@” field.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.get' timeVal #tvUsec
@
-}
getTimeValTvUsec :: MonadIO m => TimeVal -> m CLong
getTimeValTvUsec s = liftIO $ withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CLong
    return val

{- |
Set the value of the “@tv_usec@” field.
When <https://github.com/haskell-gi/haskell-gi/wiki/Overloading overloading> is enabled, this is equivalent to

@
'Data.GI.Base.Attributes.set' timeVal [ #tvUsec 'Data.GI.Base.Attributes.:=' value ]
@
-}
setTimeValTvUsec :: MonadIO m => TimeVal -> CLong -> m ()
setTimeValTvUsec s val = liftIO $ withManagedPtr s $ \ptr -> do
    poke (ptr `plusPtr` 8) (val :: CLong)

#if ENABLE_OVERLOADING
data TimeValTvUsecFieldInfo
instance AttrInfo TimeValTvUsecFieldInfo where
    type AttrAllowedOps TimeValTvUsecFieldInfo = '[ 'AttrSet, 'AttrGet]
    type AttrSetTypeConstraint TimeValTvUsecFieldInfo = (~) CLong
    type AttrBaseTypeConstraint TimeValTvUsecFieldInfo = (~) TimeVal
    type AttrGetType TimeValTvUsecFieldInfo = CLong
    type AttrLabel TimeValTvUsecFieldInfo = "tv_usec"
    type AttrOrigin TimeValTvUsecFieldInfo = TimeVal
    attrGet _ = getTimeValTvUsec
    attrSet _ = setTimeValTvUsec
    attrConstruct = undefined
    attrClear _ = undefined

timeVal_tvUsec :: AttrLabelProxy "tvUsec"
timeVal_tvUsec = AttrLabelProxy

#endif



#if ENABLE_OVERLOADING
instance O.HasAttributeList TimeVal
type instance O.AttributeList TimeVal = TimeValAttributeList
type TimeValAttributeList = ('[ '("tvSec", TimeValTvSecFieldInfo), '("tvUsec", TimeValTvUsecFieldInfo)] :: [(Symbol, *)])
#endif

-- method TimeVal::add
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "time_", argType = TInterface (Name {namespace = "GLib", name = "TimeVal"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GTimeVal", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "microseconds", argType = TBasicType TLong, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "number of microseconds to add to @time", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "g_time_val_add" g_time_val_add :: 
    Ptr TimeVal ->                          -- time_ : TInterface (Name {namespace = "GLib", name = "TimeVal"})
    CLong ->                                -- microseconds : TBasicType TLong
    IO ()

{- |
Adds the given number of microseconds to /@time_@/. /@microseconds@/ can
also be negative to decrease the value of /@time_@/.
-}
timeValAdd ::
    (B.CallStack.HasCallStack, MonadIO m) =>
    TimeVal
    {- ^ /@time_@/: a 'GI.GLib.Structs.TimeVal.TimeVal' -}
    -> CLong
    {- ^ /@microseconds@/: number of microseconds to add to /@time@/ -}
    -> m ()
timeValAdd time_ microseconds = liftIO $ do
    time_' <- unsafeManagedPtrGetPtr time_
    g_time_val_add time_' microseconds
    touchManagedPtr time_
    return ()

#if ENABLE_OVERLOADING
data TimeValAddMethodInfo
instance (signature ~ (CLong -> m ()), MonadIO m) => O.MethodInfo TimeValAddMethodInfo TimeVal signature where
    overloadedMethod _ = timeValAdd

#endif

-- method TimeVal::to_iso8601
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "time_", argType = TInterface (Name {namespace = "GLib", name = "TimeVal"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GTimeVal", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TBasicType TUTF8)
-- throws : False
-- Skip return : False

foreign import ccall "g_time_val_to_iso8601" g_time_val_to_iso8601 :: 
    Ptr TimeVal ->                          -- time_ : TInterface (Name {namespace = "GLib", name = "TimeVal"})
    IO CString

{- |
Converts /@time_@/ into an RFC 3339 encoded string, relative to the
Coordinated Universal Time (UTC). This is one of the many formats
allowed by ISO 8601.

ISO 8601 allows a large number of date\/time formats, with or without
punctuation and optional elements. The format returned by this function
is a complete date and time, with optional punctuation included, the
UTC time zone represented as \"Z\", and the /@tvUsec@/ part included if
and only if it is nonzero, i.e. either
\"YYYY-MM-DDTHH:MM:SSZ\" or \"YYYY-MM-DDTHH:MM:SS.fffffZ\".

This corresponds to the Internet date\/time format defined by
<https://www.ietf.org/rfc/rfc3339.txt RFC 3339>,
and to either of the two most-precise formats defined by
the W3C Note
<http://www.w3.org/TR/NOTE-datetime-19980827 Date and Time Formats>.
Both of these documents are profiles of ISO 8601.

Use 'GI.GLib.Structs.DateTime.dateTimeFormat' or @/g_strdup_printf()/@ if a different
variation of ISO 8601 format is required.

If /@time_@/ represents a date which is too large to fit into a @struct tm@,
'Nothing' will be returned. This is platform dependent. Note also that since
@GTimeVal@ stores the number of seconds as a @glong@, on 32-bit systems it
is subject to the year 2038 problem.

The return value of 'GI.GLib.Structs.TimeVal.timeValToIso8601' has been nullable since GLib
2.54; before then, GLib would crash under the same conditions.

/Since: 2.12/
-}
timeValToIso8601 ::
    (B.CallStack.HasCallStack, MonadIO m) =>
    TimeVal
    {- ^ /@time_@/: a 'GI.GLib.Structs.TimeVal.TimeVal' -}
    -> m (Maybe T.Text)
    {- ^ __Returns:__ a newly allocated string containing an ISO 8601 date,
   or 'Nothing' if /@time_@/ was too large -}
timeValToIso8601 time_ = liftIO $ do
    time_' <- unsafeManagedPtrGetPtr time_
    result <- g_time_val_to_iso8601 time_'
    maybeResult <- convertIfNonNull result $ \result' -> do
        result'' <- cstringToText result'
        freeMem result'
        return result''
    touchManagedPtr time_
    return maybeResult

#if ENABLE_OVERLOADING
data TimeValToIso8601MethodInfo
instance (signature ~ (m (Maybe T.Text)), MonadIO m) => O.MethodInfo TimeValToIso8601MethodInfo TimeVal signature where
    overloadedMethod _ = timeValToIso8601

#endif

-- method TimeVal::from_iso8601
-- method type : MemberFunction
-- Args : [Arg {argCName = "iso_date", argType = TBasicType TUTF8, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "an ISO 8601 encoded date string", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "time_", argType = TInterface (Name {namespace = "GLib", name = "TimeVal"}), direction = DirectionOut, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GTimeVal", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = True, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TBasicType TBoolean)
-- throws : False
-- Skip return : False

foreign import ccall "g_time_val_from_iso8601" g_time_val_from_iso8601 :: 
    CString ->                              -- iso_date : TBasicType TUTF8
    Ptr TimeVal ->                          -- time_ : TInterface (Name {namespace = "GLib", name = "TimeVal"})
    IO CInt

{- |
Converts a string containing an ISO 8601 encoded date and time
to a 'GI.GLib.Structs.TimeVal.TimeVal' and puts it into /@time_@/.

/@isoDate@/ must include year, month, day, hours, minutes, and
seconds. It can optionally include fractions of a second and a time
zone indicator. (In the absence of any time zone indication, the
timestamp is assumed to be in local time.)

Any leading or trailing space in /@isoDate@/ is ignored.

/Since: 2.12/
-}
timeValFromIso8601 ::
    (B.CallStack.HasCallStack, MonadIO m) =>
    T.Text
    {- ^ /@isoDate@/: an ISO 8601 encoded date string -}
    -> m ((Bool, TimeVal))
    {- ^ __Returns:__ 'True' if the conversion was successful. -}
timeValFromIso8601 isoDate = liftIO $ do
    isoDate' <- textToCString isoDate
    time_ <- callocBytes 16 :: IO (Ptr TimeVal)
    result <- g_time_val_from_iso8601 isoDate' time_
    let result' = (/= 0) result
    time_' <- (wrapPtr TimeVal) time_
    freeMem isoDate'
    return (result', time_')

#if ENABLE_OVERLOADING
#endif

#if ENABLE_OVERLOADING
type family ResolveTimeValMethod (t :: Symbol) (o :: *) :: * where
    ResolveTimeValMethod "add" o = TimeValAddMethodInfo
    ResolveTimeValMethod "toIso8601" o = TimeValToIso8601MethodInfo
    ResolveTimeValMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveTimeValMethod t TimeVal, O.MethodInfo info TimeVal p) => OL.IsLabel t (TimeVal -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif


